@@ -1,7 +1,8 @@
 import logging
+from typing import List
 
 from feeluown.library import AbstractProvider, ProviderV2, ProviderFlags as PF, \
-    CommentModel, BriefCommentModel, BriefUserModel, UserModel, \
+    CommentModel, BriefCommentModel, BriefUserModel, UserModel, SongModel, \
     NoUserLoggedIn, LyricModel, ModelNotFound
 from feeluown.media import Quality, Media
 from feeluown.library import ModelType, SearchType
@@ -9,6 +10,7 @@ from feeluown.utils.dispatch import Signal
 from feeluown.utils.reader import create_reader, SequentialReader
 from .api import API, CodeShouldBe200
 from .login_controller import LoginController
+from .excs import NeteaseIOError
 
 
 logger = logging.getLogger(__name__)
@@ -98,6 +100,17 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
     def current_user_fav_create_albums_rd(self):
         return create_g(self.api.user_favorite_albums, V2BriefAlbumSchema)
 
+    def current_user_fav_create_songs_rd(self):
+        if not self.has_current_user():
+            raise NoUserLoggedIn
+
+        playlists = self.current_user_list_playlists()
+        if not playlists:
+            return create_reader(())
+
+        fav_playlist = playlists[0]
+        return self.playlist_create_songs_rd(fav_playlist)
+
     def current_user_cloud_songs(self):
         return create_cloud_songs_g(
             self.api.cloud_songs,
@@ -124,13 +137,20 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
         return [_deserialize(e, V2PlaylistSchema) for e in playlists], \
             [_deserialize(e, V2PlaylistSchema) for e in fav_playlists]
 
-    def current_user_get_radio_songs(self):
+    def current_user_list_radio_songs(self, count: int = 3) -> List[SongModel]:
+        if not self.has_current_user():
+            raise NoUserLoggedIn
+
+        if count <= 0:
+            return []
+
         songs_data = self.api.get_radio_music()
-        if songs_data is None:
-            logger.error('data should not be None')
-            return None
-        return [_deserialize(song_data, V2SongSchema)
-                for song_data in songs_data]
+        if not songs_data:
+            raise NeteaseIOError('failed to fetch radio songs')
+
+        songs = [_deserialize(song_data, V2SongSchema)
+                 for song_data in songs_data]
+        return songs[:count]
 
     def rec_list_daily_playlists(self):
         # If no user login, the API return code=301.
